using System.ComponentModel.DataAnnotations;
using System.Text.Json;

namespace Expose.MyIT.Server.Models.Binder;

public class AppSettings
{

#nullable disable

    [Display(Name = "Build Number"), Required] public string BuildNumber { get; set; }
    [Display(Name = "Company"), Required] public string Company { get; set; }
    [Display(Name = "Connection String"), Required] public string ConnectionString { get; set; }
    [Display(Name = "Git Commit Seven"), Required] public string GitCommitSeven { get; set; }
    [Display(Name = "Is Development"), Required] public bool? IsDevelopment { get; set; }
    [Display(Name = "Is Staging"), Required] public bool? IsStaging { get; set; }
    [Display(Name = "MonA Resource"), Required] public string MonAResource { get; set; }
    [Display(Name = "MonA Site"), Required] public string MonASite { get; set; }
    [Display(Name = "URLs"), Required] public string URLs { get; set; }
    [Display(Name = "Working Directory Name"), Required] public string WorkingDirectoryName { get; set; }

#nullable restore

    public override string ToString()
    {
        string result = JsonSerializer.Serialize(this, new JsonSerializerOptions() { WriteIndented = true });
        return result;
    }

    private static Models.AppSettings Get(AppSettings? appSettings)
    {
        Models.AppSettings result;
        if (appSettings is null)
            throw new NullReferenceException(nameof(appSettings));
        if (appSettings.BuildNumber is null)
            throw new NullReferenceException(nameof(BuildNumber));
        if (appSettings.Company is null)
            throw new NullReferenceException(nameof(Company));
        if (appSettings.ConnectionString is null)
            throw new NullReferenceException(nameof(ConnectionString));
        if (appSettings.GitCommitSeven is null)
            throw new NullReferenceException(nameof(GitCommitSeven));
        if (appSettings.MonAResource is null)
            throw new NullReferenceException(nameof(MonAResource));
        if (appSettings.IsDevelopment is null)
            throw new NullReferenceException(nameof(IsDevelopment));
        if (appSettings.IsStaging is null)
            throw new NullReferenceException(nameof(IsStaging));
        if (appSettings.MonASite is null)
            throw new NullReferenceException(nameof(MonASite));
        if (appSettings.URLs is null)
            throw new NullReferenceException(nameof(URLs));
        if (appSettings.WorkingDirectoryName is null)
            throw new NullReferenceException(nameof(WorkingDirectoryName));
        result = new(
            appSettings.BuildNumber,
            appSettings.Company,
            appSettings.ConnectionString,
            appSettings.GitCommitSeven,
            appSettings.IsDevelopment.Value,
            appSettings.IsStaging.Value,
            appSettings.MonAResource,
            appSettings.MonASite,
            appSettings.URLs,
            appSettings.WorkingDirectoryName);
        return result;
    }

    public static Models.AppSettings Get(IConfigurationRoot configurationRoot)
    {
        Models.AppSettings result;
        AppSettings? appSettings = configurationRoot.Get<AppSettings>();
        result = Get(appSettings);
        return result;
    }

}