using Expose.MyIT.Shared.Models.Stateless;
using Microsoft.AspNetCore.Mvc.Testing;
using Microsoft.Extensions.DependencyInjection;
using Serilog;
using System.Net;

namespace Expose.MyIT.Tests;

[TestClass]
public class UnitTestClientSettingsController
{

#pragma warning disable CS8618

    private static ILogger _Logger;
    private static string _ControllerName;
    private static TestContext _TestContext;
    private static WebApplicationFactory<Server.Program> _WebApplicationFactory;

#pragma warning restore

    [ClassInitialize]
    public static void ClassInitAsync(TestContext testContext)
    {
        _TestContext = testContext;
        _Logger = Log.ForContext<UnitTestClientSettingsController>();
        _WebApplicationFactory = new WebApplicationFactory<Server.Program>();
        _ControllerName = nameof(Server.Controllers.ClientSettingsController)[..^10];
    }

    [TestMethod]
    public void TestControllerName()
    {
        _Logger.Information("Starting Web Application");
        Assert.AreEqual(IClientSettingsController<object>.GetRouteName(), _ControllerName);
        _Logger.Information($"{_TestContext?.TestName} completed");
    }

    [TestMethod]
    public void GetClientSettings()
    {
        _Logger.Information("Starting Web Application");
        IServiceProvider serviceProvider = _WebApplicationFactory.Services.CreateScope().ServiceProvider;
        IClientSettingsRepository clientSettingsRepository = serviceProvider.GetRequiredService<IClientSettingsRepository>();
        List<string> clientSettings = clientSettingsRepository.GetClientSettings(null);
        Assert.IsTrue(clientSettings is not null);
        _Logger.Information($"{_TestContext?.TestName} completed");
    }

    [TestMethod]
    public async Task GetClientSettingsApi()
    {
        HttpClient httpClient = _WebApplicationFactory.CreateClient();
        _Logger.Information("Starting Web Application");
        string actionName = nameof(IClientSettingsController<object>.Action.Client);
        HttpResponseMessage httpResponseMessage = await httpClient.GetAsync($"api/{_ControllerName}/{actionName}");
        Assert.AreEqual(HttpStatusCode.OK, httpResponseMessage.StatusCode);
        Assert.AreEqual("application/json; charset=utf-8", httpResponseMessage.Content.Headers.ContentType?.ToString());
        string result = await httpResponseMessage.Content.ReadAsStringAsync();
        httpClient.Dispose();
        Assert.IsNotNull(result);
        Assert.IsTrue(result != "[]");
        _Logger.Information($"{_TestContext?.TestName} completed");
    }

    [TestMethod]
    public void GetIpAddress()
    {
        _Logger.Information("Starting Web Application");
        IServiceProvider serviceProvider = _WebApplicationFactory.Services.CreateScope().ServiceProvider;
        IClientSettingsRepository clientSettingsRepository = serviceProvider.GetRequiredService<IClientSettingsRepository>();
        string? ipAddress = clientSettingsRepository.GetIpAddress(null);
        Assert.IsTrue(ipAddress is not null);
        _Logger.Information($"{_TestContext?.TestName} completed");
    }

    [TestMethod]
    public async Task GetIpAddressApi()
    {
        HttpClient httpClient = _WebApplicationFactory.CreateClient();
        _Logger.Information("Starting Web Application");
        string actionName = nameof(IClientSettingsController<object>.Action.IP);
        HttpResponseMessage httpResponseMessage = await httpClient.GetAsync($"api/{_ControllerName}/{actionName}");
        Assert.AreEqual(HttpStatusCode.OK, httpResponseMessage.StatusCode);
        Assert.AreEqual("text/plain; charset=utf-8", httpResponseMessage.Content.Headers.ContentType?.ToString());
        string result = await httpResponseMessage.Content.ReadAsStringAsync();
        httpClient.Dispose();
        Assert.IsNotNull(result);
        _Logger.Information($"{_TestContext?.TestName} completed");
    }

}